#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>
#include <sps2regstructs.h>
#include <assert.h>
#include <memory.h>
#include <signal.h>
#include "PS2Defines.h"
#include "ps2maths.h"
#include "ps2matrix4x4.h"
#include "pad.h"
#include "sps2wrap.h"
#include "dma.h"
#include "texture.h"
#include "font.h"

bool g_bLoop = true;

void sig_handle(int sig)
{
	g_bLoop = false;
}

int main()
{
	// Initialise control pad 0
	if(!pad_init(PAD_0, PAD_INIT_LOCK | PAD_INIT_ANALOGUE | PAD_INIT_PRESSURE))
	{
		printf("Failed to initialise control pad\n");
		pad_cleanup(PAD_0);
		exit(0);
	}	

	SPS2Manager.Initialise(4096);	// 4096 * 4K Pages = 16MB Total

	VIFStaticDMA.Initialise(1536);	// 1536 * 4K Pages = 6MB Static DMA
	VIFDynamicDMA.Initialise(256);	// 256 * 4K Pages * 2 Buffers =
									// 2MB Dynamic DMA

	// Register our signal function for every possible signal (i.e. ctrl + c)
	for(int sig=0; sig<128; sig++)
		signal(sig, sig_handle);

	// Set up the DMA packet to clear the screen. We want to clear to blue.
	SPS2Manager.InitScreenClear(0x00, 0x00, 0x00);
	
	// Load the font bitmap and data
	CFont Font;
	CTexture FontTex;
	if(!Font.Load("font.dat", true))
	{
		g_bLoop = false;
		printf("Can't load font.dat\n");
	}
	if(!FontTex.LoadBitmap("font.bmp", false, true))
	{
		printf("Can't load font.bmp\n");
		g_bLoop = false;
	}
			FontTex.Upload(TEXBUF480);
	int Frame = 0;
	
	while(g_bLoop)
	{
		VIFDynamicDMA.Fire();
		
		pad_update(PAD_0);

		SPS2Manager.BeginScene();
		

		FontTex.Select();
		
		
		// INTEGER ADD EXAMPLE
		int iAnswer, iX = 10, iY = 30;

		asm __volatile__("

			addu %0, %1, %2		# iAnswer = iX + iY

		": "=r"(iAnswer) : "r"(iX), "r"(iY));

		Font.printfL(-200,-250, 0x10, 0x80, 0x20, 0x80,
			"%d + %d = %d\n", iX, iY, iAnswer);

		// FLOATING POINT DIVIDE EXAMPLE
		float fAnswer;
		float fX = 1.0f;
		float fY = 3.0f;

		asm __volatile__("

			div.s %0, %1, %2	# fAnswer = fX / fY

		": "=f"(fAnswer) : "f"(fX), "f"(fY));

		Font.printfL(-200,-170, 0x10, 0x80, 0x20, 0x80,
			"%.1f / %.1f = %f\n", fX, fY, fAnswer);

		// FLOATING POINT DOT PRODUCT EXAMPLE
		float fV1X = 1.0f, fV1Y = 2.0f, fV1Z = 3.0f;
		float fV2X = 4.0f, fV2Y = 3.0f, fV2Z = 3.5f;

		asm __volatile__("

			mula.s %1, %4		# ACC = fV1X * fV2X
			madda.s %2, %5		# ACC = ACC + (fV1Y * fV2Y)
			madd.s %0, %3, %6	# fAnswer = ACC + (fV1Z * fV2Z)

		": "=f"(fAnswer) : "f"(fV1X), "f"(fV1Y), "f"(fV1Z), 
						   "f"(fV2X), "f"(fV2Y), "f"(fV2Z));

		Font.printfL(-200, -100, 0x10, 0x80, 0x20, 0x80,
		"| %.3f |   | %.3f |\n" \
		"| %.3f | . | %.3f | = %.3f\n" \
		"| %.3f |   | %.3f |\n",
		fV1X, fV2X,
		fV1Y, fV2Y, fAnswer,
		fV1Z, fV2Z);

		
		
		Font.printfR(  300, -235, 127, 127, 127, 127, "Frame %d", Frame++);
		
		
		SPS2Manager.EndScene();
		
		// Check for exit condition
		if((pad[0].buttons & PAD_START)&&(pad[0].buttons & PAD_SELECT)) g_bLoop = false;
	}

	pad_cleanup(PAD_0);
	
	return 0;
}

